#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@gmail.com>


"""
import os

from jaeger_client import Config
from kser.entry import EntrypointMeta
from kser.sequencing.task import Task
from opentracing import Format

TRACER = Config(service_name="kser", config={
    'sampler': {'type': 'const', 'param': 1}, 'logging': True,
    'local_agent': {'reporting_host': os.getenv('JAEGER_HOST', 'localhost')}
}).initialize_tracer()


class OpentracingBase(Task, metaclass=EntrypointMeta):
    def __init__(self, uuid=None, params=None, status="PENDING", result=None,
                 metadata=None):
        Task.__init__(self, uuid, params, status, result, metadata)
        self._span = dict()
        self.set_parent_span(
            self.metadata.get('__parent-span__', dict())
        )

    def set_parent_span(self, parent):
        self._span = parent

    def label(self, action=None):
        return "{}[{}]{}".format(
            self.__class__.__name__, self.uuid,
            " - {}".format(action) if action else ""
        )

    def create_span(self, action=None):
        span_ctx = None
        if len(self._span) != 0:
            span_ctx = TRACER.extract(Format.TEXT_MAP, self._span)

        return TRACER.start_span(self.label(action), span_ctx)

    def create_root_span(self, action=None):
        span = self.create_span(action)
        TRACER.inject(span, Format.TEXT_MAP, self._span)
        return span



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@gmail.com>


"""
import logging
import os

from kafka import KafkaConsumer
from kser import KSER_METRICS_ENABLED
from kser.controller import Controller

logger = logging.getLogger(__name__)


class Consumer(object):
    REGISTRY = Controller

    def __init__(self, config, topics):
        self.client = KafkaConsumer(**config)
        self.client.subscribe(topics)

    def run(self):
        """ Run consumer
        """
        if KSER_METRICS_ENABLED == "yes":
            from prometheus_client import start_http_server
            logger.info("Metric.Starting...")
            start_http_server(
                os.getenv("KSER_METRICS_PORT", 8888),
                os.getenv("KSER_METRICS_ADDRESS", "0.0.0.0")
            )

        logger.info("{}.Starting...".format(self.__class__.__name__))
        for msg in self.client:
            data = msg.value.decode('utf-8')
            if self.client.config['enable_auto_commit'] is False:
                self.client.commit()
                logger.debug(
                    "{}: Manual commit done.".format(self.__class__.__name__)
                )
            self.REGISTRY.run(data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@gmail.com>


"""
import logging

from kser.schemas import Message
from kser.sequencing.operation import Operation
from kser.tracing import TRACER, OpentracingBase
from opentracing import Format

logger = logging.getLogger(__name__)


class OpentracingOperation(Operation, OpentracingBase):
    def __init__(self, uuid=None, status="PENDING", params=None, tasks=None,
                 result=None, metadata=None):
        OpentracingBase.__init__(self, uuid, params, status, result, metadata)
        self.tasks = tasks or list()

    def _prerun(self):
        """ To execute before running message

        :return: Kafka message
        :rtype: kser.schemas.Message
        """
        logger.debug(self.label("PreRun"), extra=dict(kmsg=Message(
            self.uuid, entrypoint=self.__class__.path,
            params=self.params, metadata=self.metadata
        ).dump()))

        with self.create_span('PreRun') as span:
            self.check_required_params()
            self._set_status("RUNNING")
            out = self.prerun()
            span.log_kv(dict(event="done"))
            return out

    def _postrun(self, result):
        """ To execute after exection

        :param kser.result.Result result: Execution result
        :return: Execution result
        :rtype: kser.result.Result
        """
        logger.debug(self.label("PostRun"), extra=dict(kmsg=Message(
            self.uuid, entrypoint=self.__class__.path,
            params=self.params, metadata=self.metadata
        ).dump()))

        with self.create_span('PostRun') as span:
            out = self.postrun(result)
            span.log_kv(dict(event="done."))
            return out

    def unsafe_execute(self, result=None):
        with self.create_root_span("Execute") as span:
            span.log_kv(dict(event="starting..."))
            self._prerun()
            for task in self.tasks:
                span_ctx = dict()
                TRACER.inject(span, Format.TEXT_MAP, span_ctx)
                task.set_parent_span(span_ctx)
                result = task.unsafe_execute(result=result)
                if result.retcode != 0:
                    return self._onerror(result)

            result = self._onsuccess(self._postrun(result=result))
            span.log_kv(dict(event="done."))
            return result
